//
//  main.c
//  (c) 2016 - 2017
//
//  Created by Xavier Nicolay on 09/12/2015 for Rehan Noordally's research
//  Laboratoire d'Informatique et de Mathématiques
//  Université de la Réunion, France
//
// Need NXJson https://bitbucket.org/yarosla/nxjson/src

// TODO : paralleliser les accès "lecture"

#define VERSION 1.1

#include <string.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <mysql/mysql.h>

#include <gd.h>
#include "gdfontt.h"
#include "gdfonts.h"
#include "gdfontmb.h"
#include "gdfontl.h"
#include "gdfontg.h"

#define USE_TRUETYPE 1
/* #define USE_BITMAP 1 */
#define USE_TRUECOLOR 1
/* #define NO_TRUECOLOR 1 */

#include "main.h"
#include "xparse.h"
#include <dirent.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdarg.h>
// Pour la lecture de fichier JSON
#include "nxjson.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif

// va permettre de programmer la generation (ou pas de fichiers core)
#include <sys/resource.h>

///////////////////////////////////////////////
////// DEFINITION DES VARIABLES GLOBALES //////
///////////////////////////////////////////////
// Nombre de THREADS SIMULTANNES

// Tableau des tuples {@ip_src, @ip_dest}
struct xtuple *xtab = NULL;   // Initialisation du tableau
struct xtuple *xtab_j_all = NULL; // Sauvegarde tous les elements individuels JSON
u_int nTupleTab = 0;               // nombre d'entrees

struct xtuple *xtab_tmp = NULL;   // Initialisation du tableau (tmp pour la pré-analyse JSON et pour la sauvegarde des chemins --graph--)

// Liste chainee de tous les liens MPLS
struct xlien *xmpls = NULL;

// Liste chainee de tous les traces et de leurs occurences
struct occ_lien *occLien = NULL;   // Initialisation du tableau


// Descripteur du fichier de sortie
FILE *outputfile   = NULL;
FILE *outputtrace  = NULL;
FILE *outputtrajet = NULL;

// Pointeur de l'image (si traitement)
gdImagePtr im           = NULL;
FILE      *mapfileIn    = NULL;
FILE      *mapfileInOut = NULL;
int        couleurTrace;            // pour la couleur du trace
int        couleurMPLS;             // pour la couleur d'un trace MPLS
int        green;                   // pour la couleur du point final
int        blue;                    // pour la couleur de traces MPLS
int        red;                     // pour la couleur du point de depart
int        decalage = 0;            // pour decaler d'un point les traces
int        stx, sty;

// Information pour la connexion a MySQL
MYSQL     mysql;
bool      usemysql      = false;
bool      verbose       = true;

// parse le fichier mysql.ini
struct xconfig *xdbConfig;

///////////////////////////////////
////// FONCTION ELEMENTAIRES //////
///////////////////////////////////


// cette fonction transforme une chaine de caractere en nombre decimal
double xstrtod( const char *str )
{
   double v = 0;
   int    decimal = 0;
   int    j=0;
   int    n;
   double vv=0;
   char   c;
   double place = 10;
   
   n = strlen( str );
   
   // if ( ( n == 1 ) && ( str[0] == 10 ) ) n=0;
   
   // printf ("chaine bizarre : -- %c -- %d --\n", str[0], str[0] );
   while ( j < n )
   {
      c = str[j];
      j++;
      if ( c == '.' )
         decimal = 1;
      else if ( decimal == 0 )
         v = v*10 + (c - '0');
      else
      {
         vv = (c - '0') / place;
         v = v + vv;
         place *= 10;
      }
      // printf ("%c --> %f (%f)\n", c, v, vv);
   }
   
   // printf ("xstrtod : (%d) %s -> %f\n", n, str, v);
   return v;
}


// Retourne le plus grand entier entre 2 nombres
int max(int x,int y)
{
   return (x < y ? y : x);
}

void xprintf( const char * format, ...)
{
   va_list argptr;
   
   if ( verbose == true )
   {
      va_start(argptr, format);
      
      if ( outputtrace != NULL )
         vfprintf(outputtrace, format, argptr);
      else
         vprintf( format, argptr );
      va_end(argptr);
   }
}

// Affiche autant d'espace que demandé
void print_space( unsigned int sp)
{
   int i;
   for ( i=1; i<sp; i++ )
      xprintf(" ");
}

// Affiche autant d'espace que demandé
void print_diese( unsigned int sp)
{
   int i;
   for ( i=1; i<sp; i++ )
   {
      xprintf("#");
   }
}


int stricmp (const char *p1, const char *p2)
{
   register unsigned char *s1 = (unsigned char *) p1;
   register unsigned char *s2 = (unsigned char *) p2;
   unsigned char c1, c2;
   
   do
   {
      c1 = (unsigned char) toupper((int)*s1++);
      c2 = (unsigned char) toupper((int)*s2++);
      if (c1 == '\0')
      {
         return c1 - c2;
      }
   }
   while (c1 == c2);
   
   return c1 - c2;
}

unsigned int translateHTMLColor( char *colorname )
{
   unsigned int mycolor;
   
   if ( stricmp(colorname, "beige" ) == 0)
      mycolor = BEIGE;
   else if ( stricmp(colorname, "black" ) == 0)
      mycolor = BLACK;
   else if ( stricmp(colorname, "blue" ) == 0)
      mycolor = BLUE;
   else if ( stricmp(colorname, "chocolate" ) == 0)
      mycolor = CHOCOLATE;
   else if ( stricmp(colorname, "coral" ) == 0)
      mycolor = CORAL;
   else if ( stricmp(colorname, "cyan" ) == 0)
      mycolor = CYAN;
   else if ( stricmp(colorname, "darkgray" ) == 0)
      mycolor = DARKGRAY;
   else if ( stricmp(colorname, "darkkhaki" ) == 0)
      mycolor = DARKKHAKI;
   else if ( stricmp(colorname, "darkslategray" ) == 0)
      mycolor = DARKSLATEGRAY;
   else if ( stricmp(colorname, "dimgray" ) == 0)
      mycolor = DIMGRAY;
   else if ( stricmp(colorname, "gainsboro" ) == 0)
      mycolor = GAINSBORO;
   else if ( stricmp(colorname, "green" ) == 0)
      mycolor = GREEN;
   else if ( stricmp(colorname, "gold" ) == 0)
      mycolor = GOLD;
   else if ( stricmp(colorname, "gray" ) == 0)
      mycolor = GRAY;
   else if ( stricmp(colorname, "indigo" ) == 0)
      mycolor = INDIGO;
   else if ( stricmp(colorname, "khaki" ) == 0)
      mycolor = KHAKI;
   else if ( stricmp(colorname, "lavender" ) == 0)
      mycolor = LAVENDER;
   else if ( stricmp(colorname, "lightgrey" ) == 0)
      mycolor = LIGHTGREY;
   else if ( stricmp(colorname, "lightslategray" ) == 0)
      mycolor = LIGHTSLATEGRAY;
   else if ( stricmp(colorname, "lime" ) == 0)
      mycolor = LIME;
   else if ( stricmp(colorname, "limegreen" ) == 0)
      mycolor = LIMEGREEN;
   else if ( stricmp(colorname, "magenta" ) == 0)
      mycolor = MAGENTA;
   else if ( stricmp(colorname, "maroon" ) == 0)
      mycolor = MAROON;
   else if ( stricmp(colorname, "navy" ) == 0)
      mycolor = NAVY;
   else if ( stricmp(colorname, "orange" ) == 0)
      mycolor = ORANGE;
   else if ( stricmp(colorname, "pink" ) == 0)
      mycolor = PINK;
   else if ( stricmp(colorname, "red" ) == 0)
      mycolor = RED;
   else if ( stricmp(colorname, "silver" ) == 0)
      mycolor = SILVER;
   else if ( stricmp(colorname, "slategray" ) == 0)
      mycolor = SLATEGRAY;
   else if ( stricmp(colorname, "violet" ) == 0 )
      mycolor = VIOLET;
   else if ( stricmp(colorname, "white" ) == 0 )
      mycolor = WHITE;
   else if ( stricmp(colorname, "yellow" ) == 0 )
      mycolor = YELLOW;
   else
      mycolor = BLACK; // par defaut
   
   return mycolor;
}

unsigned int rgbColor( unsigned int colorNumber, char p)
{
   struct xcolor { unsigned int r; unsigned int g; unsigned int b;  };
   
   unsigned int  v=0;
   struct xcolor oneColor;
   struct xcolor allColors[] = {
      {245, 245, 220},        //  0 (BEIGE          =  2)
      {  0,   0,   0},        //  1 (BLACK          =  3)
      {  0,   0, 255},        //  2 (BLUE           =  4)
      {210, 105,  30},        //  3 (CHOCOLATE      =  5)
      {255, 127,  80},        //  4 (CORAL          =  6)
      {  0, 255, 255},        //  5 (CYAN           =  7)
      {169, 169, 169},        //  6 (DARKGRAY       =  8)
      {189, 183, 107},        //  7 (DARKKHAKI      =  9)
      { 47,  47,  79},        //  8 (DARKSLATEGRAY  = 10)
      {105, 105, 105},        //  9 (DIMGRAY        = 11)
      {220, 220, 220},        // 10 (GAINSBORO      = 12)
      {  0, 128,   0},        // 11 (GREEN          = 13)
      {255, 215,   0},        // 12 (GOLD           = 14)
      {255, 215,   0},        // 13 (GRAY           = 15)
      {128, 128, 128},        // 14 (INDIGO         = 16)
      { 75,   0,  30},        // 15 (KHAKI          = 17)
      {230, 230, 250},        // 16 (LAVENDER       = 18)
      {211, 211, 211},        // 17 (LIGHTGREY      = 19)
      {119, 136, 153},        // 18 (LIGHTSLATEGRAY = 20)
      {  0, 255,   0},        // 19 (LIME           = 21)
      { 50, 205,  50},        // 20 (LIMEGREEN      = 22)
      {255,   0, 255},        // 21 (MAGENTA        = 23)
      {128,   0,   0},        // 22 (MAROON         = 24)
      {  0,   0, 128},        // 23 (NAVY           = 25)
      {255, 165,   0},        // 24 (ORANGE         = 26)
      {255, 192, 203},        // 25 (PINK           = 27)
      {255,   0,   0},        // 26 (RED            = 28)
      {192, 192, 192},        // 27 (SYLVER         = 29)
      {112, 128, 144},        // 28 (SLATEGRAY      = 30)
      {238, 130, 238},        // 29 (VIOLET         = 31)
      {255, 255, 255},        // 30 (WHITE          = 32)
      {255, 255,   0}         // 31 (YELLOW         = 33)
                                     };

   oneColor = allColors[colorNumber - BEIGE ];
   switch ( p )
   {
      case 'r' : v = oneColor.r; break;
      case 'g' : v = oneColor.g; break;
      case 'b' : v = oneColor.b; break;
   }
   return v;
}

int isdirectory(char *path)
{
   struct stat statbuf;
   
   if (stat(path, &statbuf) == -1)
   {
      char *erreur;
      erreur = (char*) malloc( ( strlen( path) + 30 ) * sizeof (char) );
      if ( erreur == NULL )
      {
         printf("Out of memory\n");
         exit(-1);
      }
      strcpy( erreur, "while calling stat(" );
      strcat( erreur, path );
      strcat( erreur, ")");
      perror(erreur);
      return -1;
   }
   else
   {
      return S_ISDIR(statbuf.st_mode);
   }
}

int isFile(const char* name)
{
   DIR* directory = opendir(name);
   
   if(directory != NULL)
   {
      closedir(directory);
      return 0;
   }
   
   if(errno == ENOTDIR)
   {
      return 1;
   }
   
   return -1;
}

// Force la generation de fichier core
static bool EnableCoreDumps(void)
{
   struct rlimit   limit;
   
   limit.rlim_cur = RLIM_INFINITY;
   limit.rlim_max = RLIM_INFINITY;
   return setrlimit(RLIMIT_CORE, &limit) == 0;
}

//////////////////////////////////
////// FONCTION SECONDAIRES //////
//////////////////////////////////

// Ajoute dans la liste chainée, un tracé NON-DIRECTIONNEL ({x,y} -> {x,y})
// isMPLS = true : c'est un lien de type MPLS
// Et calcule le nombre d'occurence de cet evenement
int add_in_occ_lien( unsigned int xdeb, unsigned int ydeb, unsigned int xfin, unsigned int yfin, double rtt, bool isMPLS)
{
   struct occ_lien *xocclienPtr;
   struct occ_lien *newPtr;
   unsigned int     found = 0;
   
   xocclienPtr = occLien;
   while ( ( found == 0 ) && ( xocclienPtr != NULL ) )
   {
      // teste de façon bi-directionnelle
      if ( (( xocclienPtr->xdeb == xdeb ) && ( xocclienPtr->ydeb == ydeb ) && ( xocclienPtr->xfin == xfin ) && ( xocclienPtr->yfin == yfin )) ||
           (( xocclienPtr->xdeb == xfin ) && ( xocclienPtr->ydeb == yfin ) && ( xocclienPtr->xfin == xdeb ) && ( xocclienPtr->yfin == ydeb ))  )
         found = 1;
      else
         xocclienPtr = xocclienPtr->next;
   }
   
   // printf(" trouvé ? %d\n", found);
   if ( found == 1 )
   {
      xocclienPtr->occurence++;  // augmente l'occurence
      xocclienPtr->rtt_moy = rtt;
      
      xprintf(" trajet %d %d -> %d %d +1 occurence\n", xocclienPtr->xdeb, xocclienPtr->ydeb, xocclienPtr->xfin, xocclienPtr->yfin);
   }
   else
   {
      // le ptr n'a pas ete trouve ==> Ajoute
      newPtr = malloc( sizeof( struct occ_lien ) );
      if ( newPtr == NULL )
      {
         printf("Out of memory\n");
         exit(-1);
      }
      newPtr->occurence  = 1;
      newPtr->xdeb       = xdeb;
      newPtr->ydeb       = ydeb;
      newPtr->xfin       = xfin;
      newPtr->yfin       = yfin;
      newPtr->rtt_moy    = rtt;
      newPtr->mpls       = isMPLS;
      newPtr->paysDeb[0] = '\0';
      newPtr->paysFin[0] = '\0';
      newPtr->next       = occLien;
      occLien            = newPtr;
   }
   
   return ( found==1 );
}

// Ajout dans une liste chainée le tuple { hop_@IP, rtt }
int add_in_xtuple( char *srcIP, double tempo )
{
   struct xtuple *xtabPtr;
   struct xtuple *newPtr;
   int found = 0;
   
   // printf(" E: add_in_xtuple\n");
   
   xtabPtr = xtab;
   while ( ( found == 0 ) && ( xtabPtr != NULL ) )
   {
      if ( strcmp( xtabPtr->posIpStr, srcIP ) == 0 )
         found = 1;
      else
         xtabPtr = xtabPtr->next;
   }
   
   // printf(" tuple: %s found ? %d \n", srcIP, found );
   
   if ( found == 1 )
   {
      // le ptr a ete trouve
      double rtt=0;
      int    occ, ncc;
      
      occ = xtabPtr->occurence;
      xtabPtr->occurence++;  // augmente l'occurence
      ncc = xtabPtr->occurence;
      
      xprintf(" (%f * %d + %f ) / %d\n", xtabPtr->rtt_moy, occ, tempo, ncc);
      rtt = ( xtabPtr->rtt_moy * occ + tempo ) / ncc;
      xtabPtr->rtt_moy = rtt;                       // quid du rtt
      
      xprintf("   already exist : %s -- +1 (%d)\n", srcIP, xtabPtr->occurence);

   }
   else
   {
      // le ptr n'a pas ete trouve ==> Ajoute
      newPtr = malloc( sizeof( struct xtuple ) );
      if ( newPtr == NULL )
      {
         printf("Out of memory\n");
         exit(-1);
      }
      newPtr->occurence = 1;
      newPtr->rtt_moy = tempo;
      strcpy( newPtr->posIpStr, srcIP );
      newPtr->next = xtab;
      xtab = newPtr;
      
      // le ptr n'a pas ete trouve ==> Ajoute dans la liste temporaire (pour le graphique)
      newPtr = malloc( sizeof( struct xtuple ) );
      if ( newPtr == NULL )
      {
         printf("Out of memory\n");
         exit(-1);
      }
      newPtr->occurence = 1;
      newPtr->rtt_moy = tempo;
      strcpy( newPtr->posIpStr, srcIP );
      newPtr->next = xtab_tmp;
      xtab_tmp = newPtr;
   }
   
   // printf(" S: add_in_xtuple\n");
   return ( found==1 );
}

// initialise l'ouverture de la connexion MySQL
int xmysql_init()
{
   char *dbs, *dbu, *dbp, *dbdb;
   
   usemysql = false;
   mysql_init(&mysql);
   mysql_options(&mysql,MYSQL_READ_DEFAULT_GROUP,"chkroute");
   // parse_config();
   
   dbs  = search_in_config("DB_SERVEUR", xdbConfig);
   dbu  = search_in_config("DB_USER", xdbConfig);
   dbp  = search_in_config("DB_PASSWORD", xdbConfig);
   dbdb = search_in_config("DB_DATABASENAME", xdbConfig);
   
   /*
    printf ("dbs : %s\n",dbs);
    printf ("dbu : %s\n",dbu);
    printf ("dbp : %s\n",dbp);
    printf ("dbdb : %s\n",dbdb);
    */
   
   if(mysql_real_connect(&mysql, dbs, dbu, dbp, dbdb,0,NULL,0))
   {
      usemysql = true;
   }
   else
   {
      xprintf("Une erreur s'est produite lors de la connexion à la BDD! %s\n",  mysql_error(&mysql) );
   }
   
   return usemysql;
}

// Fournit une adresse IP et récupère le suffixe pays
// (dans la base MySQL)
void getPays( char *ad_ip, char *pays )
{
   MYSQL_RES   *result = NULL;
   MYSQL_ROW    row;
   my_ulonglong nb_results;
   // unsigned int num_fields;
   char         query[255];        // chaine de caractere pour construire la requete MySQL
   int          dcf;

   strcpy( query, "SELECT * FROM " );
   strcat( query, search_in_config("DB_TABLE", xdbConfig) );
   strcat( query, " WHERE IP='" );
   strcat( query, ad_ip );
   strcat( query, "'" );
   
   xprintf(" (getPays) requete sql : %s\n", query);
   mysql_query(&mysql, query);
      
   //On met le jeu de résultat dans le pointeur result
   result = mysql_store_result(&mysql);
   if ( result == NULL )
   {
      xprintf("Erreur dans l'execution de la requete (getPays) SQL : %s\n", query);
   }
   else
   {
      nb_results = mysql_num_rows(result);
      strcpy( pays, "unknown" ); // valeur par defaut
      if ( nb_results == 1 )
      {
         row = mysql_fetch_row(result);
         dcf = xparse_strtoi( search_in_config("DB_COUNTRY_FIELD", xdbConfig) );
         strncpy( pays, row[dcf], 20);
      }
      //Libération du jeu de résultat
      mysql_free_result(result);
      result=NULL;
   }
   
   xprintf(" (getPays) input %s : output %s\n", ad_ip, pays);
}

// Fournit les coordonnées X,Y de la map et récupère le suffixe pays
// (dans la base MySQL)
void getPaysFromXY( unsigned int mapx, unsigned mapy, char *pays )
{
   MYSQL_RES   *result = NULL;
   MYSQL_ROW    row;
   my_ulonglong nb_results;
   // unsigned int num_fields;
   char         query[255];               // chaine de caractere pour construire la requete MySQL
   char         ascii_coordonnees[20];
   
   // Par defaut
   strcpy ( pays, "unknown" );
   
   strcpy( query, "SELECT DISTINCT country FROM `" );
   strcat( query, search_in_config("DB_TABLE_MAP", xdbConfig) );
   strcat( query, "` WHERE floor(`mapx`)=" );
   snprintf(ascii_coordonnees, 10, "%d", mapx);
   strcat( query, ascii_coordonnees );
   strcat( query, " AND floor(`mapy`)=" );
   snprintf(ascii_coordonnees, 10, "%d", mapy);
   strcat( query, ascii_coordonnees );

   xprintf(" (getPaysXY) requete sql : %s\n", query);
   mysql_query(&mysql, query);
   
   //On met le jeu de résultat dans le pointeur result
   result = mysql_store_result(&mysql);
   if ( result == NULL )
   {
      xprintf("Erreur dans l'execution de la requete (getPays) SQL : %s\n", query);
   }
   else
   {
      nb_results = mysql_num_rows(result);
      if ( nb_results == 0 )
      {
         strcpy ( pays, "notfound" );
      }
      else
      {
         row = mysql_fetch_row(result);     // ne lit que le 1er pays retourné
         strncpy( pays, row[0], 20);
      }

      //Libération du jeu de résultat
      mysql_free_result(result);
      result=NULL;
   }
   
   xprintf(" (getPaysXY) input %d %d : output %s\n", mapx, mapy, pays);
}

// A partir de l'adresse IP, retourne les coordonnées XY de la map
// (dans la base MySQL)
void getXYFromIP( char *ad_ip, unsigned int *mapx, unsigned int *mapy )
{
   MYSQL_RES   *result = NULL;
   MYSQL_ROW    row;
   my_ulonglong nb_results;
   unsigned int num_fields;
   char         query[255];               // chaine de caractere pour construire la requete MySQL
   
   // Par defaut
   *mapx = 0;
   *mapy = 0;
   
   strcpy( query, "SELECT * FROM `" );
   strcat( query, search_in_config("DB_TABLE_MAP", xdbConfig) );
   strcat( query, "` WHERE IP='" );
   strcat( query, ad_ip );
   strcat( query, "'" );
   
   xprintf(" (getXYFromIP) requete sql : %s\n", query);
   mysql_query(&mysql, query);
   
   //On met le jeu de résultat dans le pointeur result
   result = mysql_store_result(&mysql);
   if ( result == NULL )
   {
      xprintf("Erreur dans l'execution de la requete (getXYFromIP) SQL : %s\n", query);
   }
   else
   {
      unsigned int mymax;
      int dbx, dby;
      
      dbx = xparse_strtoi( search_in_config("DB_MAP_X",xdbConfig) );
      dby = xparse_strtoi( search_in_config("DB_MAP_Y",xdbConfig) );
      // if ( dbx > dby ) max = dbx; else max = dby;
      mymax = (dbx < dby ) ? dby : dbx;
      // mymax = xmysql_max( dbx, dby);

      nb_results = mysql_num_rows(result);
      num_fields = mysql_num_fields(result);
      if ( nb_results == 0 )
      {
         // @IP inexistante
         *mapx = 0;
         *mapy = 0;
         xprintf(" (getXYFromIP) @IP inexistante : %s\n", ad_ip);

         // printf(" @IP inexistante\n");
      }
      else if ( mymax > num_fields )
      {
         // probleme de parametrage dans le fichier main.h
         xprintf(" (getXYFromIP) vous essayez de récupérer les colonnes %d %d alors que la table %s ne contient que %d champs\n", dbx, dby, search_in_config("DB_TABLE_MAP", xdbConfig), num_fields);

      }
      else
      {
         row = mysql_fetch_row(result);     // ne lit que le 1er pays retourné
         *mapx = xstrtod( row[ dbx ] );
         *mapy = xstrtod( row[ dby ] );
      }
      
      //Libération du jeu de résultat
      mysql_free_result(result);
      result=NULL;
   }
   
   xprintf(" (getXYFromIP) input %s : output %d %d\n", ad_ip, *mapx, *mapy);
}

void add_mpls_link( char *adip1, char *adip2 )
{
   struct xlien *xlink;
   unsigned int  found = 0;
   unsigned int  mapx1, mapy1, mapx2, mapy2;
   
   // printf(" entre dans add_mpls_link with %s et %s\n", adip1, adip2);
   
   // récupère les coordonnées geographiques du lien
   getXYFromIP( adip1, & mapx1, & mapy1 );
   getXYFromIP( adip2, & mapx2, & mapy2 );
   
   // printf(" recupère les coordoonnées %s (%d %d) et %s (%d %d)\n", adip1, mapx1, mapy1, adip2, mapx2, mapy2);

   if ( ( mapx1 != 0 ) && ( mapx2 != 0 ) && ( mapy1 != 0 ) && ( mapy2 != 0 ) )
   {
      xlink = xmpls;
      while ( ( found == 0 ) && ( xlink != NULL ) )
      {
         // teste de façon bi-directionnelle
         if ( ( (xlink->mapx1 == mapx1) && (xlink->mapy1 == mapy1) && (xlink->mapx2 == mapx2) && (xlink->mapy2 == mapy2) ) ||
             ( (xlink->mapx1 == mapx2) && (xlink->mapy1 == mapy2) && (xlink->mapx2 == mapx1) && (xlink->mapy2 == mapy1) ) )
            found = 1;
         else
            xlink = xlink->next;
      }
   }
   
   if ( found == 0 )
   {
      // le ptr n'a pas ete trouve ==> Ajoute
      struct xlien *newPtr;
      
      newPtr = malloc( sizeof( struct xlien ) );
      if ( newPtr == NULL )
      {
         xprintf("Out of memory\n");
         exit(-1);
      }
      newPtr->mapx1 = mapx1;
      newPtr->mapy1 = mapy1;
      newPtr->mapx2 = mapx2;
      newPtr->mapy2 = mapy2;
      strcpy( newPtr->IpStr1, adip1 );
      strcpy( newPtr->IpStr2, adip2 );
      newPtr->typeLien = TL_MPLS;
      newPtr->next = xmpls;
      xmpls = newPtr;
      
      xprintf(" NEW MPLS link %d %d <--> %d %d\n", mapx1, mapy1, mapx2, mapy2);
   }
   
   
   /*
    xlink = xmpls;
    while ( ( found == 0 ) && ( xlink != NULL ) )
    {
    // teste de façon bi-directionnelle
    if ( ( ( strcmp( xlink->IpStr1, adip1) == 0) && ( strcmp( xlink->IpStr2, adip2) == 0) ) &&
    ( ( strcmp( xlink->IpStr1, adip2) == 0) && ( strcmp( xlink->IpStr2, adip1) == 0) ) )
    found = 1;
    else
    xlink = xlink->next;
    }
    
    if ( found == 0 )
    {
    // le ptr n'a pas ete trouve ==> Ajoute
    struct xlien *newPtr;
    
    newPtr = malloc( sizeof( struct xlien ) );
    if ( newPtr == NULL )
    {
    printf("Out of memory\n");
    exit(-1);
    }
    strcpy( newPtr->IpStr1, adip1 );
    strcpy( newPtr->IpStr2, adip2 );
    newPtr->typeLien = TL_MPLS;
    newPtr->next = xmpls;
    xmpls = newPtr;
    
    xprintf(" NEW MPLS link %s <--> %s\n", adip1, adip2);
    }
    */
}

// Retourne vrai si le trace {(x1,y1) (x2,y2)} est un trace MPLS
//bool isMPLSlink( char *adip1, char *adip2 )
bool isMPLSlink( unsigned int mapx1, unsigned int mapy1, unsigned int mapx2, unsigned int mapy2 )
{
   struct xlien *xlink;
   bool mpls = false;
   
   // printf(" --isMPLSlink : IN : %s et %s\n", adip1, adip2);
   
   xlink = xmpls;
   while ( ( ! mpls) && ( xlink != NULL ) )
   {
      // printf(" --isMPLSlink : %s et %s\n", xlink->IpStr1, xlink->IpStr2);

      // teste de façon bi-directionnelle
      // if ( ( ( strcmp( xlink->IpStr1, adip1) == 0) && ( strcmp( xlink->IpStr2, adip2) == 0) ) || ( ( strcmp( xlink->IpStr1, adip2) == 0) && ( strcmp( xlink->IpStr2, adip1) == 0) ) )
      if ( ( (xlink->mapx1 == mapx1) && (xlink->mapy1 == mapy1) && (xlink->mapx2 == mapx2) && (xlink->mapy2 == mapy2) ) ||
          ( (xlink->mapx1 == mapx2) && (xlink->mapy1 == mapy2) && (xlink->mapx2 == mapx1) && (xlink->mapy2 == mapy1) ) )
         mpls = true;
      else
         xlink = xlink->next;
   }
   
   return mpls;
}

// Cette fonction, utilisee en sortie, affiche le contenu de la liste de tous les tuples en memoire
// De facon optionnelle, on fera une requete sur une base MySQL pour recuperer la geolocalisation
// d'adresse IP.
int list_xtuple( struct xtuple *xtabRef )
{
   struct xtuple *xtabPtr = NULL;
   char pos[10];           // tableau pour recopier la position (avant le '_')
   char *pos_under;        // pointeur vers le caractere '_'
   char *s_ptr_ad_ip;      // pointeur pour l'@IP
   int  sdiff;             // longeur de la chaine de caractere avant le '_'
   char sfloat[20];        // chaine de caractere pour afficher le nombre flottant
   char geopos[200];       // chaine de caractere pour la geolocalisation
   // unsigned int xstrlen=0;
   // unsigned int num_fields;

   xtabPtr = xtabRef;
   while ( xtabPtr != NULL )
   {
      // xstrlen = strlen( xtabPtr->posIpStr );
      if (  xtabPtr->posIpStr[0] == 10 )
         xtabPtr->posIpStr[0] = ' ';

      if ( xtabPtr->occurence > 0 )
      {
         pos_under = strchr(xtabPtr->posIpStr, '_' );
         if ( pos_under != NULL )
         {
            s_ptr_ad_ip = pos_under;
            s_ptr_ad_ip++;

            sdiff = strlen(xtabPtr->posIpStr) - strlen( pos_under );
            memset( pos, 0, 9);
            strncpy(pos, xtabPtr->posIpStr, sdiff );
            // printf ("%s -> %s %s (%lu) (%lu) %d\n", xtabPtr->posIpStr, pos, s_ptr_ad_ip , strlen(xtabPtr->posIpStr), strlen( pos_under ), sdiff);
            if ( xtabPtr->rtt_moy == 0 )
               strcpy( sfloat, "0");
            else
               sprintf(sfloat, "%f", xtabPtr->rtt_moy);
            
            if ( usemysql )
               getPays( s_ptr_ad_ip, geopos );
            else
               strcpy( geopos, "unknown" );

            if ( verbose == false )
               ;
            else if ( outputfile != NULL )
            {
               fprintf ( outputfile, "%4s %20s %4d %15s %s\n", pos, s_ptr_ad_ip, xtabPtr->occurence, sfloat, geopos);
            }
            else
               printf ("%4s %20s %4d %15s %s\n", pos, s_ptr_ad_ip, xtabPtr->occurence, sfloat, geopos);
            
         }
      }
      xtabPtr = xtabPtr->next;
   }
   
   return 0;
}


/* Teste si le fichier termine par 3 lignes "* * *"
   Retourne 1 si c'est le cas (fichier inutile)
   returne 0 si le fichier est "utile"
 */
int isGoodFile(char* filename)
{
   FILE *fp;
   long fsize;
   char buffer[256]="";
   int  nbStar = 0;
   char lastChar1, lastChar2, lastChar3;
   int  len;
   int  ignore = 0;
   
   fp = fopen( filename, "r");
   fseek(fp, 0, SEEK_END);      // seek to end of file
   fsize = ftell(fp);           // get current file pointer

   // 1er test
   // Recherche si les 3 dernieres lignes contiennent uniquement des étoiles
   fseek(fp, (fsize - 60), SEEK_SET);
   while( !feof( fp ) )
   {
      fgets( buffer , sizeof buffer , fp );
      len = strlen( buffer );
      if ( len > 7 )
      {
         lastChar1 = buffer[ len - 2 ];
         lastChar2 = buffer[ len - 4 ];
         lastChar3 = buffer[ len - 6 ];
         if ( ( lastChar1 == '*' ) && ( lastChar2 == '*' ) && ( lastChar3 == '*' ) )
            nbStar++;
         if ( nbStar == 3 )
         {
            xprintf("fichier inutile : %s\n", filename);
            ignore=1;
         }
      }
      // printf ("ligne : %s --> (%d %c)\n", buffer, len, lastChar);
   }
   
   // 2nd test
   // Regarde si le fichier contient !H ou !N ou s'il est "incohérent" (contient 2 fois la ligne traceroute par exemple)
   if ( ignore == 0 )
   {
      char        *pos          = NULL;
      unsigned int nbtraceroute = 0;

      fseek(fp, 0, SEEK_SET);
      while ( (fgets(buffer, sizeof(buffer), fp) != NULL) && (ignore==0) )
      {
         if ((pos = strstr(buffer, "traceroute")) != NULL)
         {
            nbtraceroute++;
            if ( nbtraceroute > 1 )
               ignore = 1;
         }
         else if ((pos = strstr(buffer, "!H")) != NULL)
            ignore = 1;
         else if ((pos = strstr(buffer, "!N")) != NULL)
            ignore = 1;
         else if ((pos = strstr(buffer, "WARN")) != NULL)
            ignore = 1;
      }
      if ( ignore == 1 )
      {
         xprintf("fichier inutile : %s\n", filename);
      }
   }
   fclose (fp);
   return ignore;
}

/* Ouvre le fichier passe en parametre, de type paris-traceroute pour l'inclure dans l'analyse */
int comptabiliseFile(  char* filename )
{
   int    ret=0;
   FILE  *fd;
   char   ligne[256];
   char   posStr[256];
   char   ipStr[256];
   char   tempoStr[256];
   int    firstline=0;
   char   c, last_c;
   int    lenPosStr;
   int    lenIpStr;
   int    lenTempoStr;
   int    nbTempo;
   char   posIp[256];
   char   phrase[1024];
   double moy, tempo1, tempo2, tempo3;
   char   lastIp[50];
   int    startMPLS=0;
   
   fd = fopen( filename , "r");
   if ( fd )
   {
      int j;
      int jSuite=0;
      char *pos;
      
      xprintf("   fopen : %s\n", filename);

      while (fgets(ligne, 256, fd) != NULL)
      {
         // printf(" analyse : %s\n", ligne );
         if ( firstline == 0 )
         {
            pos = strstr(ligne, "traceroute");
            if ( pos != NULL )
               firstline = 1;
         }
         else if ( firstline == 1 )
         {
            // Ne prend pas en compte (en charge) les lignes contenant MPLS
            pos = strstr(ligne, "MPLS");
            if ( pos != NULL )
            {
               startMPLS=1;
               continue;
            }

            // Ne prend pas en compte (en charge) les lignes contenant Bandwidth
            pos = strstr(ligne, "Bandwidth");
            if ( pos != NULL )
               continue;
            
            // ne prends pas en compte les lignes "vides"
            if ( ( strlen( ligne ) == 1 ) && (ligne[0] == 10 ) )
               continue;

            // recherche la position
            j=0;
            lenPosStr=0;
            posStr[lenPosStr] = '\0';
            while ( j < strlen(ligne) )
            {
               c = ligne[j];
               if ( ( c == ' ' ) && ( lenPosStr == 0 ) )  // ne compte pas les espaces en debut de ligne
                  j++;
               else if ( c == ' ' )                       // on tombe sur le 1er espace ne demarrant pas la ligne
               {
                  jSuite = j;
                  j = strlen(ligne); // sors de la boucle
               }
               else
               {
                  posStr[lenPosStr] = c;       // memorise le caractere
                  lenPosStr++;                 // marque la fin de la chaine
                  posStr[lenPosStr] = '\0';
                  j++;
               }
            }
            
            // printf("fin de la recherche position \n");
            
            // recherche l'@IP
            j=jSuite;
            lenIpStr=0;
            ipStr[lenIpStr] = '\0';
            while ( j < strlen(ligne) )
            {
               c = ligne[j];
               if ( ( ( c == ' ' ) || ( c == '*' ) ) && ( lenIpStr == 0 ) )  // ne compte pas les espaces ou * en debut de ligne
                  j++;
               else if ( c == ' ' )                       // on tombe sur le 1er espace ne demarrant pas la ligne
               {
                  jSuite = j;
                  j = strlen(ligne); // sors de la boucle
               }
               else
               {
                  ipStr[lenIpStr] = c;       // memorise le caractere
                  lenIpStr++;                 // marque la fin de la chaine
                  ipStr[lenIpStr] = '\0';
                  j++;
               }
            }
            
            if ( (strlen( ipStr ) > 0 ) && ( startMPLS == 1 ) )
            {
               // Ouvre un lien MPLS entre 2 @IP
               add_mpls_link( lastIp, ipStr );
               startMPLS = 0;
            }
            strcpy( lastIp, ipStr );

            // printf("fin de la recherche IP %s\n", ipStr);

            // recherche les temps
            tempo1=0; tempo2=0; tempo3=0; moy=0;
            nbTempo = 0;
            j=jSuite;
            lenTempoStr=0;
            tempoStr[lenTempoStr] = '\0';
            last_c = '\0';
            while ( j < strlen(ligne) )
            {
               c = ligne[j];
               if ( ( ( c == ' ' ) || ( c==10 ) || ( c == '*' ) || ( c == 'm' ) || ( c == 's' ) || ( c == '!' ) || ( c == 'T' ) || ( ( c == '0' ) && ( last_c == 'T') ) ) && ( lenTempoStr == 0 ) )  // ne compte pas les espaces ou * en debut de ligne
                  j++;
               else if ( c == ' ' )                       // on tombe sur le 1er espace ne demarrant pas la ligne
               {
                  jSuite = j;
                  j = strlen(ligne); // sors de la boucle
               }
               else
               {
                  tempoStr[lenTempoStr] = c;       // memorise le caractere
                  lenTempoStr++;                   // marque la fin de la chaine
                  tempoStr[lenTempoStr] = '\0';
                  j++;
               }
               last_c = c;
            }
            // printf("fin de la recherche tempo1 %s : %d\n", tempoStr, jSuite);

            if ( ( lenTempoStr > 0 ) && ( jSuite < strlen(ligne) ) )
            {
               // printf("calcul de tempo1 (%s)\n", tempoStr);
               // tempo1 = strtod( tempoStr, pEnd );
               tempo1 = xstrtod( tempoStr );
               // printf("[%s -> %f] / calcul de tempo1\n", tempoStr, tempo1);
               nbTempo++;
               j=jSuite;
               lenTempoStr=0;
               tempoStr[lenTempoStr] = '\0';
               last_c = '\0';
               while ( j < strlen(ligne) )
               {
                  c = ligne[j];
                  if ( ( ( c == ' ' ) || ( c==10) || ( c == '*' ) || ( c == 'm' ) || ( c == 's' ) || ( c == '!' ) || ( c == 'T' ) || ( ( c == '0' ) && ( last_c == 'T') ) ) && ( lenTempoStr == 0 ) )  // ne compte pas les espaces ou * en debut de ligne
/*
                     if ( ( ( c == ' ' ) || ( c == '*' ) ||
                         ( c == 'm' ) || ( c == 's' ) ||
                         ( c == '!' ) || ( c == 'T' ) ||
                         ( ( c == '0' ) && ( last_c == 'T' ) ) ) && ( lenTempoStr == 0 ) )  // ne compte pas les espaces ou * en debut de ligne
*/
                     j++;
                  else if ( c == ' ' )                       // on tombe sur le 1er espace ne demarrant pas la ligne
                  {
                     jSuite = j;
                     j = strlen(ligne); // sors de la boucle
                  }
                  else
                  {
                     tempoStr[lenTempoStr] = c;       // memorise le caractere
                     lenTempoStr++;                   // marque la fin de la chaine
                     tempoStr[lenTempoStr] = '\0';
                     // printf("new str : %s\n", tempoStr);
                     j++;
                  }
                  last_c = c;
               }
               // printf("fin de la recherche tempo2\n");
               
               if ( ( lenTempoStr > 0 ) && ( jSuite < strlen(ligne ) ) )
               {
                  // tempo2 = strtod( tempoStr, pEnd );
                  
                  tempo2 = xstrtod( tempoStr );
                  // printf("[%s -> %f] / calcul de tempo2\n", tempoStr, tempo2);
                  nbTempo++;
                  j=jSuite;
                  lenTempoStr=0;
                  tempoStr[lenTempoStr] = '\0';
                  last_c = '\0';
                  while ( j < strlen(ligne) )
                  {
                     c = ligne[j];
                     if ( ( ( c == ' ' ) || ( c==10) || ( c == '*' ) || ( c == 'm' ) || ( c == 's' ) || ( c == '!' ) || ( c == 'T' ) || ( ( c == '0' ) && ( last_c == 'T') ) ) && ( lenTempoStr == 0 ) )  // ne compte pas les espaces ou * en debut de ligne
                        j++;
                     else if ( c == ' ' )                       // on tombe sur le 1er espace ne demarrant pas la ligne
                     {
                        jSuite = j;
                        j = strlen(ligne); // sors de la boucle
                     }
                     else
                     {
                        tempoStr[lenTempoStr] = c;       // memorise le caractere
                        lenTempoStr++;                   // marque la fin de la chaine
                        tempoStr[lenTempoStr] = '\0';
                        j++;
                     }
                     last_c = c;
                  }
                  
                  if ( ( lenTempoStr > 0 ) && ( jSuite < strlen(ligne ) ) )
                  {
                     // tempo3 = strtod( tempoStr, pEnd );
                     tempo3 = xstrtod( tempoStr );
                     nbTempo++;
                  }
               }
               // printf ("tempoStr(%s) -> %f\n", tempoStr, tempo1);
            }
            
            // printf ("l1 = %d, l2 = %d\n", lenPosStr, lenIpStr);
            
            if ( ( lenPosStr > 0 ) && ( lenIpStr > 1 ) )
            {
               strcpy( posIp, posStr );
               strcat( posIp, "_" );
               strcat( posIp, ipStr );
               strcpy( phrase, "infos trouvees : " );
               strcat( phrase, posStr);
               strcat( phrase, " et ");
               strcat( phrase, ipStr);
               strcat( phrase, " --> ");
               strcat( phrase, posIp);
               strcat( phrase, " --- tempoStr = ");
               // printf ("presque la %d\n", nbTempo);
               moy = 0;
               switch ( nbTempo )
               {
                  case 0 : {
                     moy = 0;
                     strcpy( phrase, posIp );
                     strcat( phrase, " ignore ");
                     strcat( phrase, ligne);
                     
                  }
                  case 1 : {
                     // printf("tempoStr = %f", tempo1);
                     // snprintf(s, sizeof s, "n = %d, x = %.6f", n, x);
                     
                     sprintf( ligne, "%f", tempo1);
                     strcat ( phrase, ligne );
                     moy = tempo1;
                     break;
                  }
                  case 2 : {
                     // printf("tempoStr = %f et %f", tempo1, tempo2);
                     sprintf( ligne, "%f et %f", tempo1, tempo2);
                     strcat ( phrase, ligne );
                     moy = ( tempo1 + tempo2 ) / 2;
                     break;
                  }
                  case 3 : {
                     // printf("tempoStr = %f et %f et %f", tempo1, tempo2, tempo3);
                     sprintf( ligne, "%f et %f et %f", tempo1, tempo2, tempo3);
                     strcat ( phrase, ligne );
                     moy = ( tempo1 + tempo2 + tempo3) / 3;
                     break;
                  }
               }
               if ( moy > 0 )
               {
                  sprintf( ligne, " [%f]", moy);
                  strcat ( phrase, ligne );
               }
               
               xprintf ("%s\n", phrase );
               add_in_xtuple( posIp, moy );
            }
            else
            {
               // traite les lignes "14 * * *"
               strcpy( posIp, posStr );
               strcat( posIp, "_*" );
               moy=0;
               add_in_xtuple( posIp, moy );
            }

         }
      }
      
      fclose(fd);
   }
   return ret;
}

/* Vide le contenu de la liste temporaire xtab_tmp */
int destroy_json_tmp()
{
   if ( xtab_tmp != NULL )
   {
      struct xtuple *xtabPtr;
      struct xtuple *xtabFree;
      xtabPtr = xtab_tmp;
      while ( xtabPtr != NULL )
      {
         xtabFree = xtabPtr;
         xtabPtr = xtabPtr->next;
         free(xtabFree);
         xtabFree=NULL;
      }
      xtab_tmp = NULL;
   }

   if ( xtab_j_all != NULL )
   {
      struct xtuple *xtabPtr;
      struct xtuple *xtabFree;
      xtabPtr = xtab_j_all;
      while ( xtabPtr != NULL )
      {
         xtabFree = xtabPtr;
         xtabPtr = xtabPtr->next;
         free(xtabFree);
         xtabFree=NULL;
      }
      xtab_j_all = NULL;
   }

   return 0;
}

// Affiche toutes les occurences des tracés
unsigned int print_occ_lien()
{
   unsigned int     maxocc = 0;
   unsigned int     n = 0;

   if ( occLien != NULL )
   {
      struct occ_lien *xocclienPtr;
      
      if ( outputtrajet )
         fprintf ( outputtrajet, "# { x,y } <-> { x,y }  #occ\n");

      xocclienPtr = occLien;
      while ( xocclienPtr != NULL )
      {
         xprintf("   ++ sur le trajet {%d,%d} -> {%d,%d} : %d\n", xocclienPtr->xdeb, xocclienPtr->ydeb, xocclienPtr->xfin, xocclienPtr->yfin, xocclienPtr->occurence);
         if ( outputtrajet )
         {
            fprintf ( outputtrajet, "trajet %03d {%d,%d} -> {%d,%d} : %d\n", n, xocclienPtr->xdeb, xocclienPtr->ydeb, xocclienPtr->xfin, xocclienPtr->yfin, xocclienPtr->occurence );
            n++;
         }
         maxocc = max( maxocc, xocclienPtr->occurence );
         xocclienPtr = xocclienPtr->next;
      }
   }
   
   return maxocc;
}

// calcule la matrice des trajet inter-pays
void calcule_matrice()
{
   int nbPays = 0;
/*
   if ( occLien != NULL )
   {
      struct occ_lien *xocclienPtr;
      xocclienPtr = occLien;
      while ( xocclienPtr != NULL )
      {
         
      }
   }
*/
}



// Affiche toutes les occurences des tracés
// (retrace les liens selon l'epaisseur liée aux occurences)
int draw_occ_lien( unsigned int maxocc )
{
   if ( occLien != NULL )
   {
      struct occ_lien *xocclienPtr;
      // struct occ_lien *xoccFree;
      unsigned int thickness;
      unsigned int couleur;
      float a,b;
      
      a = (MAX_THICKNESS - 1 ) / (float) (maxocc -1);
      b = 1 - a;
      xocclienPtr = occLien;
      while ( xocclienPtr != NULL )
      {
         thickness = a * xocclienPtr->occurence + b;
         // printf(" thickness = %d\n", thickness );
         gdImageSetThickness ( im, thickness );
         couleur = ( xocclienPtr->mpls ) ? couleurMPLS : couleurTrace;
         // printf(" trajet %d %d - %d %d = %d\n", xocclienPtr->xdeb+decalage, xocclienPtr->ydeb+decalage, xocclienPtr->xfin+decalage, xocclienPtr->yfin+decalage, xocclienPtr->mpls );

         couleur = couleurMPLS;

         if ( (( xocclienPtr->xdeb > 3000 ) && ( xocclienPtr->xfin < 1400 )) || (( xocclienPtr->xdeb < 1400 ) && ( xocclienPtr->xfin > 3000 )) )
         {
            unsigned int finx, finy;
            finx = 4500;
            finy = ( xocclienPtr->ydeb + xocclienPtr->yfin ) / 2;
            if ( xocclienPtr->xdeb > xocclienPtr->xfin )
            {
               gdImageLine(im, xocclienPtr->xdeb+decalage, xocclienPtr->ydeb+decalage, finx, finy, couleur);
               gdImageLine(im, 0, finy, xocclienPtr->xfin+decalage, xocclienPtr->yfin+decalage, couleur);
            }
            else
            {
               gdImageLine(im, xocclienPtr->xfin+decalage, xocclienPtr->yfin+decalage, finx, finy, couleur);
               gdImageLine(im, 0, finy, xocclienPtr->xdeb+decalage, xocclienPtr->ydeb+decalage, couleur);
            }
         }
         else
            gdImageLine(im, xocclienPtr->xdeb+decalage, xocclienPtr->ydeb+decalage, xocclienPtr->xfin+decalage, xocclienPtr->yfin+decalage, couleur);

         xocclienPtr = xocclienPtr->next;
      }
   }
   
   return maxocc;
}


/* Sort proprement du programme en liberant la memoire utilisee */
int sortie()
{
  
   if ( xtab != NULL )
   {
      struct xtuple *xtabPtr;
      struct xtuple *xtabFree;
      xtabPtr = xtab;
      while ( xtabPtr != NULL )
      {
         xtabFree = xtabPtr;
         xtabPtr = xtabPtr->next;
         free(xtabFree);
         xtabFree=NULL;
      }
      xtab = NULL;
   }

   if ( occLien != NULL )
   {
      struct occ_lien *xocclienPtr;
      struct occ_lien *xoccFree;
      xocclienPtr = occLien;
      while ( xocclienPtr != NULL )
      {
         xoccFree = xocclienPtr;
         xocclienPtr = xocclienPtr->next;
         free(xoccFree);
         xoccFree=NULL;
      }
      occLien = NULL;
   }
   
   if ( xmpls != NULL )
   {
      struct xlien *xlink;
      struct xlien *xlinkFree;
      char   pays1[20];
      char   pays2[20];
      xlink = xmpls;
      while ( xlink != NULL )
      {
         xlinkFree = xlink;
         xlink = xlink->next;
         
         getPays( xlinkFree->IpStr1, pays1);
         getPays( xlinkFree->IpStr2, pays2);
         xprintf ("  // lien <-> MPLS : %s %s\n", xlinkFree->IpStr1, xlinkFree->IpStr2);
         free(xlinkFree);
         xlinkFree=NULL;
      }
      xmpls = NULL;
   }

   destroy_json_tmp();
   
   if ( usemysql )
      mysql_close(&mysql);

   if ( im != NULL )
      gdImageDestroy(im);
   
   if ( mapfileIn != NULL )
      fclose(mapfileIn);
   if ( outputfile != NULL )
      fclose (outputfile);
   if ( outputtrace != NULL )
      fclose (outputtrace);
   if ( outputtrajet != NULL )
      fclose (outputtrajet);
   return 0;
}

int add_in_json( int hop, char *srcIP, double tempo, unsigned int decale )
{
   struct xtuple *xtabPtr;
   struct xtuple *newPtr;
   
   int found = 0;
   char newSrc[60];
   
   snprintf(newSrc, 60, "%d_%s", hop, srcIP);
   
   // printf(" E: add_in_xtuple\n");
   
   // Sauvegarde systematique dans *xtab_j_all
   newPtr = malloc( sizeof( struct xtuple ) );    // reservation memoire
   if ( newPtr == NULL )
   {
      printf("Out of memory\n");
      exit(-1);
   }
   newPtr->occurence = 1;                         // sauvegarde elementaire
   newPtr->rtt_moy = tempo;                       // sauvegarde de la tempo
   strcpy( newPtr->posIpStr, newSrc );            // sauvegarde du saut et de l'@IP
   newPtr->next = xtab_j_all;                     // liste chainee
   xtab_j_all = newPtr;                           // nouveau pointeur de debut de liste
   // Sauvegarde systematique dans *xtab_j_all -- end
   
   xtabPtr = xtab_tmp;
   while ( ( found == 0 ) && ( xtabPtr != NULL ) )
   {
      if ( strcmp( xtabPtr->posIpStr, newSrc ) == 0 )
         found = 1;
      else
         xtabPtr = xtabPtr->next;
   }
   
   if ( found == 1 )
   {
      // le ptr a ete trouve
      double rtt=0;
      int    occ, ncc;
      
      occ = xtabPtr->occurence;
      xtabPtr->occurence++;  // augmente l'occurence
      ncc = xtabPtr->occurence;
      
      print_space( decale);
      xprintf(" (%f * %d + %f ) / %d\n", xtabPtr->rtt_moy, occ, tempo, ncc);
      rtt = ( xtabPtr->rtt_moy * occ + tempo ) / ncc;
      xtabPtr->rtt_moy = rtt;                       // quid du rtt
      
      print_space( decale);
      xprintf("   already exist : %s -- +1 (%d)\n", newSrc, xtabPtr->occurence);
      
   }
   else
   {
      // le ptr n'a pas ete trouve ==> Ajoute
      newPtr = malloc( sizeof( struct xtuple ) );
      if ( newPtr == NULL )
      {
         xprintf("Out of memory\n");
         exit(-1);
      }
      newPtr->occurence = 1;
      newPtr->rtt_moy = tempo;
      strcpy( newPtr->posIpStr, newSrc );
      newPtr->next = xtab_tmp;
      
      xtab_tmp = newPtr;
   }
   return ( found==1 );
}

// Analyse récursivement toute la structure JSON
int parse_jsonAO( const nx_json* jp, int prof_recursion, int result_level )
{
   nx_json    *js;
   static char ad_ip[50];
   static char src_addr[50];
   static char dst_addr[50];
   static char icmp_ext[255];
   static char msm_name[50];     // Traceroute
   static char dst_name[255];    // rpi105.noordally.com
   static char proto[20];        // ICMP
   static char ip_start[50];
   static double rtt=0.0;
   static int psize=0;
   static int decale;
   static int start_mpls_link=0;
   static int hop=0;
   static int af=0;
   static int endtime=0;
   static int fw=0;
   static int group_id=0;
   static int ittl=0;
   static int late=0;
   static int lts=0;
   static int msm_id=0;
   static int paris_id=0;
   static int prb_id=0;
   static int rfc4884=0;
   static int version=0;
   static int mpls_exp=0;
   static int mpls_label=0;
   static int mpls_s=0;
   static int mpls_type=0;
   static int mpls_class=0;
   static int timestamp=0;
   static int ttl=0;
   // int result_level=0;
   
   if ( jp == NULL )
   {
      // fin de la recursion
      printf ("jp NULL\n");
      return hop;
   }
   // else printf ("jp non-null %u et %u\n", prof_recursion, hop);

   if ( (prof_recursion == 0 ) && ( hop == 0 ) )
   {
      // Initialisation
      ad_ip[0]    = '\0';
      ip_start[0] = '\0';
      rtt=-1;
      // next_ip_will_be_mpls=0;
      // start_mpls_link = 0;
      result_level    = 0;
      start_mpls_link = 0;
   }

   decale = prof_recursion * 3;

   // print_space ( decale ); if ( jp->type == NX_JSON_STRING )  printf ("Entree : hop = %d - key : %s %s\n", hop, jp->key, jp->text_value);

   // if ( hop==10 ) exit(127);
   
   switch (jp->type)
   {
      case NX_JSON_NULL:    // 0
         break;
      case NX_JSON_OBJECT:  // 1
      {
         if ( jp->key )
         {
            print_space ( decale ); xprintf ("Name : %s is an Object (prof_recurs = %d)\n", jp->key,prof_recursion+1 );
         }
         // else { print_space ( decale ); printf("obj_start (decale = %d)\n", decale); }
         if ( jp->child )
         {
            js=jp->child;
            while ( js != NULL )
            {
               // printf ("appel jparseAO(js, %u, %u)\n", prof_recursion+1, result_level );
               parse_jsonAO( js, prof_recursion+1, result_level );
               js=js->next;
            }
            // for (js=jp->child; js; js=js->next) parse_jsonAO( js, prof_recursion+1, result_level );
            decale -= 3;
         }
         // print_space ( decale ); printf("obj_end (decale = %d)\n", decale);
         break;
      }
      case NX_JSON_ARRAY:  // 2
      {
         if ( jp->key != NULL )
         {
            // print_space ( decale ); printf ("Name : %s is an Array (prof_recurs = %d)\n", jp->key,prof_recursion+1 );
            print_space ( decale ); xprintf( "Name : %s is an Array (prof_recurs = %d)\n", jp->key,prof_recursion+1 );
            if ( ! strcmp( jp->key , "result" ) )
               result_level++;
            if ( ! strcmp( jp->key , "mpls" ) )
            {
               // printf(" result level : %d et start_mpls_link = %d et ad_ip = %s et ip_start = %s\n", result_level, start_mpls_link, ad_ip, ip_start);

               if ( strlen( ip_start ) == 0 )
               {
                  // demarre le lien MPLS
                  strcpy( ip_start, ad_ip);
                  start_mpls_link = 1;
                  print_diese ( decale ); xprintf ("Memorise %s\n", ip_start);
               }
            }
            else if ( start_mpls_link == 1 )
            {
               start_mpls_link = 2;
            }
         }
         
         if ( jp->child != NULL )
         {
            js=jp->child;
            for (js=jp->child; js; js=js->next)
            {
               parse_jsonAO( js, prof_recursion+1, result_level );
               if ( jp->key )
               {
                  if ( ! strcmp( jp->key, "result") )
                  {
                     if ( result_level == 2 )
                     {
                        // char c = ( start_mpls_link > 0 ) ? 'M' : ' ';
                        // print_space ( decale );  printf("[%c] go to analyse %d (%d)\n", c, i++, result_level);
                        add_in_json( hop, ad_ip, rtt, decale );
                     }
                  }
               }
            }
         }
         break;
      }
      case NX_JSON_STRING:  // 3
      {
         if ( jp->key )
         {
            if ( ! strcmp( jp->key, "from") )
            {
               print_space ( decale );
               xprintf(" __%d__ Result FROM : %s = %s\n", hop, jp->key, jp->text_value );
               
               strcpy( ad_ip, jp->text_value );
               if ( start_mpls_link == 2 )
               {
                  if ( strcmp( ip_start, ad_ip))
                  {
                     print_diese ( decale ); xprintf ("cloture (MPLS) %s - %s\n", ip_start, ad_ip );
                     // lien entre l'ancienne @IP et l'actuelle
                     add_mpls_link( ip_start, ad_ip );
                     // redemarre le compteur a zero
                     start_mpls_link = 0;
                     ip_start[0]='\0';
                  }
               }
            }
            else if ( ! strcmp( jp->key, "src_addr") )
            {
               print_space ( decale );
               xprintf(" __%d__ Result SRC_ADDR : %s = %s\n", hop, jp->key, jp->text_value );
               strcpy( src_addr, jp->text_value );
            }
            else if ( ! strcmp( jp->key, "dst_addr") )
            {
               print_space ( decale );
               xprintf(" __%d__ Result DST_ADDR : %s = %s\n", hop, jp->key, jp->text_value );
               strcpy( dst_addr, jp->text_value );
            }
            else if ( ! strcmp( jp->key, "dst_name") )
            {
               print_space ( decale );
               xprintf( " __%d__ Result DST_NAME : %s = %s\n", hop, jp->key, jp->text_value );
               strcpy( dst_name, jp->text_value );
            }
            else if ( ! strcmp( jp->key, "msm_name") )
            {
               print_space ( decale );
               xprintf(" __%d__ Result MSM_NAME : %s = %s\n", hop, jp->key, jp->text_value );
               strcpy( msm_name, jp->text_value );
            }
            else if ( ! strcmp( jp->key, "proto") )
            {
               print_space ( decale );
               xprintf(" __%d__ Result PROTO : %s = %s\n", hop, jp->key, jp->text_value );
               strcpy( proto, jp->text_value );
            }
            else if ( ! strcmp( jp->key, "icmpext") )
            {
               if ( jp->text_value == NULL )
                  strcpy( icmp_ext, "none" );
               else
                  strcpy( icmp_ext, jp->text_value );

               print_space ( decale );
               xprintf(" __%d__ Result ICMP_EXT : %s = %s\n", hop, jp->key, icmp_ext );
            }
/*
            else if ( ! strcmp( jp->key, "mpls") )
            {
               // Demarre un lien MPLS
               next_ip_will_be_mpls=1;
               start_mpls_link = 1;
               
               print_space ( decale );
               xprintf(" __%d__ MPLS START\n", hop ); // d = 4
            }
*/
            else if ( ! strcmp( jp->key, "x") && ! strcmp( jp->text_value, "*") )
            {
               // ignore this value
               ;
            }
            else if ( ! strcmp( jp->key, "result") && (jp->text_value == NULL ) )
               ;
            else if ( ! strcmp( jp->key, "obj") && (jp->text_value == NULL ) )
               ;
            else if ( ! strcmp( jp->key, "type") && ! strcmp( jp->key, "type") && ! strcmp( jp->text_value, "traceroute"))
               ;
            else
            {
               print_space ( decale );
               xprintf ("######## %s = %s (string)\n", jp->key, jp->text_value );
            }
         }
         break;
      }
      case NX_JSON_INTEGER:  // 4
      {
         if ( jp->key )
         {
            if ( ! strcmp( jp->key, "hop") )
            {
               hop = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result HOP : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "size") )
            {
               psize = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result SIZE : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "rfc4884") )
            {
               rfc4884 = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result rfc4884 : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "version") )
            {
               version = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result version : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "s") )
            {
               mpls_s = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result mpls_s : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "exp") )
            {
               mpls_exp = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result mpls_exp : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "label") )
            {
               mpls_label = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result mpls_label : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "class") )
            {
               mpls_class = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result mpls_class : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "af") )
            {
               af = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result AF : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "endtime") )
            {
               endtime = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result EndTime : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "fw") )
            {
               fw = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result FW : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "group_id") )
            {
               group_id = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result group_id : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "ittl") )
            {
               ittl = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result ITTL : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
            }
            else if ( ! strcmp( jp->key, "lts") )
            {
               lts = jp->int_value;
               print_space ( decale );
               xprintf(" __%d__ Result LTS : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
         }
         else if ( ! strcmp( jp->key, "late") )
         {
            late = jp->int_value;
            print_space ( decale );
            xprintf(" __%d__ Result LATE : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
         }
         else if ( ! strcmp( jp->key, "msm_id") )
         {
            msm_id = jp->int_value;
            print_space ( decale );
            xprintf(" __%d__ Result MSM_id : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
         }
         else if ( ! strcmp( jp->key, "paris_id") )
         {
            paris_id = jp->int_value;
            print_space ( decale );
            xprintf(" __%d__ Result PARIS_id : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
         }
         else if ( ! strcmp( jp->key, "prb_id") )
         {
            prb_id = jp->int_value;
            print_space ( decale );
            xprintf(" __%d__ Result PRB_id : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
         }
         else if ( ! strcmp( jp->key, "timestamp") )
         {
            timestamp = jp->int_value;
            print_space ( decale );
            xprintf(" __%d__ Result timestamp : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
         }
         else if ( ! strcmp( jp->key, "ttl") )
         {
            ttl = jp->int_value;
            print_space ( decale );
            xprintf(" __%d__ Result TTL : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
         }
         else if ( ! strcmp( jp->key, "type") )
         {
            mpls_type = jp->int_value;
            print_space ( decale );
            xprintf(" __%d__ Result mpls_type : %s = %lld\n", hop, jp->key, jp->int_value ); // d = 4
         }
            else
            {
              print_space ( decale );
              xprintf ("######## %s = %lld (int)\n", jp->key, jp->int_value );
            }
         }
         break;
      }
      case NX_JSON_DOUBLE:  // 5
      {
         if ( ! strcmp( jp->key, "rtt") )
         {
            rtt = jp->dbl_value;
            print_space ( decale );
            xprintf(" __%d__ Result RTT : %s %f\n", hop, jp->key, jp->dbl_value ); // d = 4
         }
         else
         {
            print_space ( decale );
            xprintf ("######## %s = %f (double)\n", jp->key, jp->dbl_value );
         }
         break;
      }
      case NX_JSON_BOOL:    // 6
      {
         char v = jp->int_value?'T':'F';
         print_space ( decale );
         xprintf ("%s = %c (bool)\n", jp->key, v );
         break;
      }
      default:
      {
         print_space ( decale );
         xprintf ("Warning hop %d ... type : %d\n",hop, jp->type );
         break;
      }
   }

   /*
         else if ( ! strcmp( js->key, "x") )
         {
            print_space ( decale );
            if ( verbose == false )
               ;
            else if ( outputtrace != NULL )
               fprintf(outputtrace, " __%d__ Result : x %s\n", hop, js->text_value );
            else
               printf(" __%d__ Result : x %s\n", hop, js->text_value );
            // add_in_json( hop, (*j)->u.s, 0, decale );
         }
   }
   */
   // print_space ( decale ); xprintf ("Sortie avec hop=%d\n", hop);
   return prof_recursion;
}

// Lance l'analyse récursive des données JSON (déjà parsées)
int analyse_json( const nx_json* jp )
{
   int ret=-1;
   int lasthop=0;
   
   // parse_jsonAO( jp );
   
   // printf(" entree dans l'analyse JSON\n");
   if ( jp == NULL )
   {
      xprintf ("Erreur in JSON file\n");
   }
   else
   {
      lasthop = parse_jsonAO( jp, 0, 0);
      xprintf("nombre total de saut : %d\n", lasthop);
   }
   // printf(" sortie dans l'analyse JSON\n");

   return ret;
}

// Valide le fichier JSON (dont le nom est passé en parametre, à partir du fichier déjà parsé et temporairement dans la variable xtab_tmp
int isValideJsonParsing( char* filename)
{
   bool ret = false;
   char rep[10];
   struct xtuple *xtabPtr = NULL;
   // char *pos_under;        // pointeur vers le caractere '_'
   // char *s_ptr_ad_ip;      // pointeur pour l'@IP
   double rtt_2, rtt_1, rtt_0;
   xprintf("------------------------------------------\n");
   rtt_2 = 0;
   rtt_1 = 0;
   rtt_0 = 0;
   xtabPtr = xtab_tmp;
   if ( xtabPtr != NULL )
   {
      rtt_0 = xtabPtr->rtt_moy;
      xtabPtr = xtabPtr->next;
      if ( xtabPtr != NULL )
      {
         rtt_1 = xtabPtr->rtt_moy;
         xtabPtr = xtabPtr->next;
         if ( xtabPtr != NULL )
         {
            rtt_2 = xtabPtr->rtt_moy;
         }
      }
   }
   
   xprintf(" rtt-2 : %f\n", rtt_2 );
   xprintf(" rtt-1 : %f\n", rtt_1 );
   xprintf(" rtt-0 : %f\n", rtt_0 );
   if ( ( rtt_0 == 0 ) && ( rtt_1 == 0 ) && ( rtt_2 == 0 ) )
   {
      xprintf(" --> fichier (%s) inutile\n", filename);
   }
   else
      ret = true;
   
   strcpy( rep, "BAD" );
   if ( ret )
      strcpy( rep, "GOOD" );
   
   xprintf("-------------------- %s ------------------\n", rep);
   return ret;
}

int json_validate()
{
   struct xtuple *xtabPtr = NULL;
   double rtt;
   char  *srcIP;
   
   // parse la sauvegarde (elementaire)
   xtabPtr = xtab_j_all;
   while ( xtabPtr != NULL )
   {
      rtt   = xtabPtr->rtt_moy;
      srcIP = xtabPtr->posIpStr;
      add_in_xtuple( srcIP, rtt );
      xtabPtr = xtabPtr->next;
   }
   return 0;
}

void debug_data( struct xtuple *xtabinfos )
{
   struct xtuple *xtabPtr = NULL;
   
   xtabPtr = xtabinfos;
   while ( xtabPtr != NULL )
   {
      xprintf("    -->%s\n", xtabPtr->posIpStr);
      xtabPtr = xtabPtr->next;
   }
}

// A partir de la liste constituee, recherche tous les liens pointant sur {stx, sty}
// Utilisée pour dessinée la carte des derniers liens (glast)
void recherche_connexions( int stx, int sty, gdImagePtr im )
{
   // printf(" E: recherche connexions\n");
   // par precaution
   if ( ( occLien != NULL ) && ( im != NULL ) )
   {
      struct occ_lien *xocclienPtr;
      unsigned int     totalOccurence=0;
      unsigned int     firstx, firsty;
      unsigned int     couleur;
      // unsigned int     stopx, stopy;
      float            ratio;
      float            thickness;
      char             pays[200];
      char             paysdeb[30];
      char             paysfin[30];
      bool             loop;
      bool             isMPLS;
      char             rtt_str[30];
      char             ratio_str[20];

      
      xocclienPtr = occLien;
      while ( xocclienPtr != NULL )
      {
         if ( (( xocclienPtr->xdeb == stx ) && ( xocclienPtr->ydeb == sty )) || (( xocclienPtr->xfin == stx) && ( xocclienPtr->yfin == sty )) )
         {
            // printf ("  _p1_ trajet : %d %d -> %d %d : %d\n", xocclienPtr->xdeb, xocclienPtr->ydeb, xocclienPtr->xfin, xocclienPtr->yfin, xocclienPtr->occurence );
            totalOccurence += xocclienPtr->occurence;
         }
         xocclienPtr = xocclienPtr->next;
      }
      // printf ("  _p1_ total des occurences entrantes : %d\n", totalOccurence );
      
      firstx=firsty=0;
      loop = ( totalOccurence > 0 );
      xocclienPtr = occLien;
      while ( ( xocclienPtr != NULL ) && (loop) )
      {
         rtt_str[0]=0;
         if ( (( xocclienPtr->xdeb == stx ) && ( xocclienPtr->ydeb == sty )) || (( xocclienPtr->xfin == stx) && ( xocclienPtr->yfin == sty )) )
         {
            isMPLS = isMPLSlink( xocclienPtr->xdeb, xocclienPtr->ydeb, xocclienPtr->xfin, xocclienPtr->yfin );
            ratio = 100 * xocclienPtr->occurence / (float) totalOccurence;
            snprintf( ratio_str, 20, "%.2f", ratio);
            couleur = (isMPLS) ? couleurMPLS : couleurTrace;
            thickness = ratio / (float) 10;
            if ( thickness < 1 )
               thickness = 1;
            gdImageSetThickness ( im, thickness );
            
            if (( xocclienPtr->xdeb == stx ) && ( xocclienPtr->ydeb == sty ))
            {
               firstx = xocclienPtr->xfin;
               firsty = xocclienPtr->yfin;
               // stopx  = stx;
               // stopy  = sty;
            }
            else
            {
               firstx = xocclienPtr->xdeb;
               firsty = xocclienPtr->ydeb;
               // stopx  = xocclienPtr->xfin;
               // stopy  = xocclienPtr->yfin;
            }
            gdImageFilledEllipse(im, firstx+decalage, firsty+decalage, 20, 20, red);
            
            snprintf( (char*) rtt_str, 20, "%.02f", xocclienPtr->rtt_moy);
            
            if ( (( xocclienPtr->xdeb > 3000 ) && ( xocclienPtr->xfin < 1400 )) || (( xocclienPtr->xdeb < 1400 ) && ( xocclienPtr->xfin > 3000 )) )
            {
                  int finx, finy;
                  
                  finx = 4500;
                  finy = ( xocclienPtr->ydeb + xocclienPtr->yfin ) / 2;
                  if ( xocclienPtr->xdeb > xocclienPtr->xfin )
                  {
                     gdImageLine(im, xocclienPtr->xdeb+decalage, xocclienPtr->ydeb+decalage, finx, finy, couleur);
                     gdImageLine(im, 0, finy, xocclienPtr->xfin+decalage, xocclienPtr->yfin+decalage, couleur);
                  }
                  else
                  {
                     gdImageLine(im, xocclienPtr->xfin+decalage, xocclienPtr->yfin+decalage, finx, finy, couleur);
                     gdImageLine(im, 0, finy, xocclienPtr->xdeb+decalage, xocclienPtr->ydeb+decalage, couleur);
                  }
            }
            else
            {
               unsigned int  midx, midy;
               int brect[8];
               char *err;
               float coeff_dir;
               float xtan;
               // float angdegres;
               // int mymodulo;
               // int myzero;
               int px, py;

               midx = ( xocclienPtr->xdeb + xocclienPtr->xfin ) / 2;
               midy = ( xocclienPtr->ydeb + xocclienPtr->yfin ) / 2;
               
               px = (xocclienPtr->xfin - xocclienPtr->xdeb );
               py = (xocclienPtr->ydeb - xocclienPtr->yfin ); // les y sont inversés
               coeff_dir =  py / (float) px;
               xtan = atan(coeff_dir);
               // printf("----------------------------------\n");
               // printf ("texte : %s\n", rtt_str);
               // printf(" xfin - xdeb = %d - %d = %d\n", xocclienPtr->xfin, xocclienPtr->xdeb, px );
               // printf(" ydeb - yfin = %d - %d = %d\n", xocclienPtr->ydeb, xocclienPtr->yfin, py );
               // if ( xtan != 0 )
               // {
               //   angdegres = RAD2DEG( xtan );
                  // printf("pente = %.2f et cotan(pente) = %.2f = %.2f°\n", coeff_dir, xtan, angdegres );
                  
                  // if ( angdegres > -80)
                  // angdegres = 18.77;
               // }
               
               strcat (rtt_str, " (");
               strcat (rtt_str, ratio_str);
               strcat (rtt_str, "%)");
               gdImageLine(im, xocclienPtr->xdeb+decalage, xocclienPtr->ydeb+decalage, xocclienPtr->xfin+decalage, xocclienPtr->yfin+decalage, couleur);
               midx= midx + thickness + 2;
               err = gdImageStringTTF( im, &brect[0], couleur, FONT_FILE, FONT_SIZE, xtan, midx, midy, rtt_str );
               if (err)
               {
                  fprintf(stderr,"erreur de fonte (?): %s\n", err);
               }
            }
            
            getPaysFromXY( xocclienPtr->xdeb, xocclienPtr->ydeb, paysdeb);
            strncpy( xocclienPtr->paysDeb, paysdeb, 10);
            strcpy ( pays, paysdeb );
            strcat ( pays, " - " );
            getPaysFromXY( xocclienPtr->xfin, xocclienPtr->yfin, paysfin);
            strncpy( xocclienPtr->paysFin, paysfin, 10);
            strcat ( pays, paysfin );

            // strcat ( pays, " " );
            if ( xocclienPtr->mpls )
               strcat ( pays, " MPLS");
            
            if ( strlen (rtt_str) > 0 )
            {
               strcat ( pays, " et temps min ");
               strcat ( pays, rtt_str);
            }
            
            xprintf ("  __ trajet : %d %d -> %d %d : %d (%0.2f%%) [%s]\n", xocclienPtr->xdeb, xocclienPtr->ydeb, xocclienPtr->xfin, xocclienPtr->yfin, xocclienPtr->occurence, ratio, pays );
         }
         xocclienPtr = xocclienPtr->next;
      }
      gdImageFilledEllipse(im, stx+decalage, sty+decalage, 20, 20, green);
   }
   // printf(" S: recherche connexions\n");
}

// Dessine TOUS les liens à partir des informations contenues (pré-parsées) dans la liste passée en argument
int draw_data( struct xtuple *xtabinfos )
{
   struct xtuple *xtabPtr = NULL;
   unsigned int   lastx = 0;
   unsigned int   lasty = 0;
   char          *pos_under;        // pointeur vers le caractere '_'
   char          *s_ptr_ad_ip;      // pointeur pour l'@IP
   char           lastIp[50];
   double         rtt_xy, last_rtt=0;
   unsigned int   mapx, mapy;
   // unsigned int   dx, dy;
   // bool           isMPLS;
   // double         distance;
   // float          acceptance;
   
   // parse la sauvegarde (elementaire)
   xtabPtr = xtabinfos;
   lastIp[0]=0;
   while ( xtabPtr != NULL )
   {
      pos_under = strchr(xtabPtr->posIpStr, '_' );
      if ( pos_under != NULL )
      {
         s_ptr_ad_ip = pos_under;
         s_ptr_ad_ip++;
         
         if ( strncmp( s_ptr_ad_ip, "*", 1) == 0 )
            ;
         else if ( strncmp( s_ptr_ad_ip, "192.168.", 8) == 0 )
            ;
         else if ( strncmp( s_ptr_ad_ip, "10.", 3) == 0 )
            ;
         else if ( strncmp( s_ptr_ad_ip, "172.16.0.", 9) == 0 )
            ;
         else if ( strncmp( s_ptr_ad_ip, "172.16.1", 8) == 0 )
            ;
         else if ( strncmp( s_ptr_ad_ip, "172.16.2", 8) == 0 )
            ;
         else if ( strncmp( s_ptr_ad_ip, "172.16.30.", 10) == 0 )
            ;
         else if ( strncmp( s_ptr_ad_ip, "172.16.31.", 10) == 0 )
            ;
         else
         {
            getXYFromIP( s_ptr_ad_ip, & mapx, & mapy );
            
            // printf("   trace de  : %d %d -> %d %d\n", lastx, lasty, mapx, mapy);
            // printf(" lecture OK done\n");
            if ( ( mapx !=0 ) && ( mapy!=0 ) )
            {
               rtt_xy = fabs ( last_rtt - xtabPtr->rtt_moy );
               // printf(" rtt : |%0.2f - %0.2f| = %0.2f\n", last_rtt, xtabPtr->rtt_moy, rtt_xy);

               if ( ( lastx == 0 ) && ( lasty == 0 ) )
                  gdImageFilledEllipse(im, mapx+decalage, mapy+decalage, 20, 20, red);
               else if ( ( lastx != mapx ) && ( lasty != mapy ) )
               {
                  int  couleur;
                  bool isMPLS;
                  char txtb[20];
                        
                  // isMPLS = isMPLSlink( lastIp, s_ptr_ad_ip);
                  isMPLS = isMPLSlink( lastx, lasty, mapx, mapy);

                  couleur = (isMPLS) ? couleurMPLS : couleurTrace;
                  if ( isMPLS )
                     strcpy (txtb, "[trace en MPLS]");
                  else
                     strcpy (txtb, "");

                  xprintf(" (graph) trace de (%s) : %d %d -> (%s) %d %d %s\n", lastIp, lastx, lasty, s_ptr_ad_ip, mapx, mapy, txtb);
                        
                  // printf("   trace de  : %d %d <-> %d %d (%s %s) %s\n", mapx, mapy, lastx, lasty, lastIp, s_ptr_ad_ip, txtb );
                  
                  if ( (( lastx > 3000 ) && ( mapx < 1400 )) || (( lastx < 1400 ) && ( mapx > 3000 )) )
                  {
                     int finx, finy;
                           
                     finx = 4500;
                     finy = ( lasty + mapy ) / 2;
                     if ( lastx > mapx )
                     {
                        gdImageLine(im, lastx+decalage, lasty+decalage, finx, finy, couleur);
                        gdImageLine(im, 0, finy, mapx+decalage, mapy+decalage, couleur);
                     }
                     else
                     {
                         gdImageLine(im, mapx+decalage, mapy+decalage, finx, finy, couleur);
                         gdImageLine(im, 0, finy, lastx+decalage, lasty+decalage, couleur);
                     }
                  }
                  else
                     gdImageLine(im, mapx+decalage, mapy+decalage, lastx+decalage, lasty+decalage, couleur);
                     // printf(" add occ lien %d %d %d %d\n", x, y, lastx, lasty );
                  add_in_occ_lien( mapx, mapy, lastx, lasty, rtt_xy, isMPLS);
                  
/*
                  dx = abs( (int) mapx - (int) lastx );   // typage int pour éviter les warnings
                  dy = abs( (int) mapy - (int) lasty );   // typage int pour éviter les warnings
                  distance = sqrt( dx*dx + dy*dy );    // version optimisee de " distance = sqrt( pow(dx,2) + pow(dy,2) ); "
*/                  
                  if ( isMPLS )
                     strcpy (txtb, "M");
                  else
                     strcpy (txtb, " ");
                  
                  /*
                  acceptance = distance / 200;
                  printf ("compare %7.02f et %7.02f -- ", acceptance, rtt_xy );
                  if ( rtt_xy < acceptance )
                     strcpy( warn, "warning - ");
                  else
                     strcpy( warn, "          ");
                  getPays( lastIp, pays1);
                  getPays( s_ptr_ad_ip, pays2);
                  
                  // Partie a debugger / affiner
                  printf ("%slienDistance (%-15s <-> %-15s) : (%-6s <-> %-6s) :%s: ({%4d,%4d} - {%4d %4d}) = %7.02f -- rtt = %7.02f\n", warn, lastIp, s_ptr_ad_ip, pays1, pays2, txtb, mapx, mapy, lastx, lasty, distance, rtt_xy);
                  */
               }
                     
               lastx    = mapx;
               lasty    = mapy;
               last_rtt = xtabPtr->rtt_moy;
               strcpy( lastIp, s_ptr_ad_ip );
            }
         }
      }
      xtabPtr = xtabPtr->next;
   }

   if (( lastx == 0 ) && ( lasty == 0 ))
   {
      xprintf ("Toute la liste est vide !!!\n");
   }
   else
      gdImageFilledEllipse(im, lastx+decalage, lasty+decalage, 20, 20, green);
   return 0;
}

nx_json* jparse( const char* filepath )
{
   struct stat st;
   nx_json* json;
   
   if (stat(filepath, &st)==-1)
   {
      xprintf("can't access file %s\n", filepath);
      return NULL;
   }
   
   int fd=open(filepath, O_RDONLY);
   if ( fd == -1 )
   {
      xprintf("can't open file %s\n", filepath);
      return NULL;
   }

   char* text=malloc(st.st_size+1); // this is not going to be freed
   if (st.st_size != read(fd, text, st.st_size))
   {
      xprintf("can't read file %s\n", filepath);
      close(fd);
      return NULL;
   }
   close(fd);
   text[st.st_size]='\0';

   json=nx_json_parse_utf8(text);
   free(text);
   text = NULL;

   if (!json)
   {
       return NULL;
   }
   else
   {
      return json;
   }
}

/* mode d'emploi du programme !! :) */
int usage( char* fn)
{
   printf("Usage : %s [-s] [-j] -p repDatafiles [-d outputfile] [-gin map.png] [-gout new_map.png] [-c colorname] [-dec] [-redraw] [-stx valx] [-sty valy] [-t trajectfile] [-rej]\n", fn);
   printf("   -s                    : mode très silencieux\n");
   printf("   -j                    : fichiers au format JSON (optionnel)\n");
   printf("   -p repDatafiles       : repertoire contenant les fichiers à analyser\n");
   printf("   -d outputfile         : fichier resultat issu de l'analyse\n");
   printf("   -gin map.png          : (entrée) carte mondiale (vierge) pour le tracé des flux (png ou jpg)\n");
   printf("   -gout world_new.png   : (sortie) carte mondiale pour le tracé des flux (world_new.png par defaut)\n");
   printf("   -glast world_last.png : (sortie) carte mondiale pour le tracé des derniers flux (world_last.png par defaut)\n");
   printf("   -c colorname          : couleur (html) du tracé {beige, black, blue, red, green, white, yellow, ...}\n");
   printf("   -mpls colorname       : couleur (html) du tracé pour les liens MPLS {beige, black, blue, red, green, white, yellow, ...}\n");
   printf("   -dec                  : decale les traces d'un point\n");
   printf("   -redraw               : retrace les liens avec l'épaisseur des occurences d'usages\n");
   printf("   -stx valx             : analyse tous les tracés convergents vers ces coordonnées (valx ET valy)\n");
   printf("   -sty valy             : analyse tous les tracés convergents vers ces coordonnées (valx ET valy)\n");
   printf("   -v                    : affiche la version\n");
   printf("   -core                 : (debug) autorise la generation d'un fichier core\n");
   printf("   -rej                  : ajouter l'extention '.rej' aux fichiers incorrects\n");
   printf("   -t trajectfile        : sauvegarde tous les trajets, et les occurrences des chemins empruntés\n");
   return 0;
}

/******************
 ****** MAIN ******
 ******************/
int main(int argc, char** argv)
{
   int                result=0;
   float              temps;
   clock_t            t1=0, t2=0;
   char              *repName = NULL;
   // void              *status;
   int                i;
   DIR               *rep            = NULL;
   unsigned int       uSage          = 1;       // erreur par defaut
   int                argUtile       = 0;
   int                error          = 0;
   int                mycolor        = RED;
   int                mplscolor      = BLUE;
   bool               json_data      = false;
   bool               redraw         = false;
   bool               graphFile      = false;
   bool               version        = false;
   bool               matrice        = false;
   bool               markRejFile    = false;
   char              *mapfilename    = NULL;
   char              *strmapfileout  = NULL;
   char              *strmapfilelast = NULL;
   char              *posPNG         = NULL;
   char              *posJPEG        = NULL;
   char              *mysqlfile      = NULL;
   unsigned int       nb_files        = 0;

   unsigned int       mypid;
   
   mypid = getpid();
   
   mysqlfile = (char*) malloc( 10 );
   if ( mysqlfile == NULL )
   {
      printf("Out of memory\n");
      exit(-1);
   }
   // fichier mysql.ini par defaut
   strcpy( mysqlfile, "mysql.ini" );
   
   stx = -1;
   sty = -1;
   // printf("argc : %d\n", argc);
   for ( i=1; i<argc; i++ )
   {
      if ( ( strcmp( argv[i], "-p" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         // printf ("Top repertoire d'analyse : %s\n", argv[i]);
         rep = opendir ( argv[i] );
         if ( rep == NULL )
         {
            printf("impossible d'ouvrir le répertoire %s\n", argv[i]);
            error = 1;
         }
         else
         {
            repName = (char*) argv[i];
         }
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-stx" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         // printf ("fichier resultat d'analyse : %s\n", argv[i]);
         stx = xstrtod( argv[i] );
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-sty" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         // printf ("fichier resultat d'analyse : %s\n", argv[i]);
         sty = xstrtod( argv[i] );
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-m" ) == 0 ) && ( (i+1) < argc ) )
      {
         // printf ("fichier resultat d'analyse : %s\n", argv[i]);
         int n;

         i++;
         n = strlen( argv[i] ) + 1;
         // printf ("taille demandee : %u\n", n);
         
         mysqlfile = (char*) realloc (mysqlfile, strlen( argv[i] ) + 1 );
         if ( mysqlfile == NULL )
         {
            printf("realloc : Out of memory\n");
            exit(-1);
         }
         // printf ("memoire OK\n");
         strcpy( mysqlfile, argv[i] );
         // printf ("copie OK\n");
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-d" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         // printf ("fichier resultat d'analyse : %s\n", argv[i]);
         outputfile = fopen( argv[i], "w" );
         outputtrace = fopen( "trace.txt", "w" );
         // repName = (char*) argv[i];
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-t" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         // printf ("fichier des trajets : %s\n", argv[i]);
         outputtrajet = fopen( argv[i], "w" );
         // repName = (char*) argv[i];
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-gin" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         // printf ("fichier resultat d'analyse : %s\n", argv[i]);
         mapfileIn = fopen( argv[i], "r" );
         if ( mapfileIn )
         {
            mapfilename = argv[i];
         }
         else
         {
            printf("impossible d'ouvrir le fichier %s\n", argv[i]);
            error = 1;
         }
         argUtile++;
         graphFile = true;
         // repName = (char*) argv[i];
      }
      else if ( ( strcmp( argv[i], "-gout" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         strmapfileout = argv[i];
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-glast" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         strmapfilelast = argv[i];
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-c" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         // printf ("couleur du tracé : %s\n", argv[i]);
         mycolor = translateHTMLColor( argv[i] );
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-mpls" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         // printf ("couleur du tracé : %s\n", argv[i]);
         mplscolor = translateHTMLColor( argv[i] );
         argUtile++;
      }
      else if ( ( strcmp( argv[i], "-rej" ) == 0 ) && ( (i+1) < argc ) )
      {
         i++;
         markRejFile = true;
         argUtile++;
      }
      else if ( strcmp (argv[i], "-dec" ) == 0 )
      {
         argUtile++;
         decalage += 1;
      }
      else if ( strcmp (argv[i], "-redraw" ) == 0 )
      {
         argUtile++;
         redraw = true;
      }
      else if ( strcmp (argv[i], "-j" ) == 0 )
      {
         argUtile++;
         json_data = true;
      }
      else if ( strcmp (argv[i], "-s" ) == 0 )
      {
         argUtile++;
         verbose = false;
      }
      else if ( strcmp (argv[i], "-matrice" ) == 0 )
      {
         argUtile++;
         matrice = true;
      }
      else if ( strcmp (argv[i], "-v" ) == 0 )
      {
         argUtile++;
         version = true;
         verbose = false;
      }
      else if ( strcmp (argv[i], "-core" ) == 0 )
      {
         argUtile++;
         printf("création possible de fichier core\n");
         EnableCoreDumps();
      }
   }
   
   file_ini_parse( mysqlfile, &xdbConfig );
   if ( xdbConfig == NULL )
   {
      error = 1;
      fprintf( stderr, "file mysql.ini : not found or useless\n");
   }
   else
   {
      if ( ! xmysql_init() )
      {
         error = 1;
         fprintf( stderr, "Acces mysql impossible\n");
      }
   }
   
   if ( error == 1 )
   {
      usage( argv[0] );
      exit(EXIT_FAILURE);
   }
   
   if ( verbose == false )
      ;
   else if ( outputtrace )
   {
      fprintf (outputtrace, "argUtile = %d\n", argUtile );
      fprintf(outputtrace, "run with : ");
      for ( i=0; i<argc; i++ )
         fprintf(outputtrace, "%s ", argv[i] );
      fprintf(outputtrace, "\n");
   }
   else
      printf ("argUtile = %d\n", argUtile );
   
   if ( verbose )
      printf("PID du process : %u\n", mypid);

   if ( argUtile == 0 )
   {
      uSage = 1;
   }
   else
      uSage = 0;

   if ( uSage == 1 )
   {
      usage( argv[0] );
      result = 1;
   }
   else if ( version )
   {
      printf ("Version %.1f\n", VERSION );
      result = 0;
      verbose = false;
   }
   else if ( rep == NULL )
   {
      // il est obligatoire de fournir un repertoire a analyser
      usage( argv[0] );
      result = 1;
   }
   else
   {
      struct dirent     *lecture;
      struct stat       *st_niv0;
      struct stat       *st_niv1;
      char               nfile_niv0[1024];
      nx_json           *JSA;
      
      t1 = clock();
      
      st_niv0 = (struct stat *) malloc(sizeof(struct stat));
      st_niv1 = (struct stat *) malloc(sizeof(struct stat));
      
      if ( ( st_niv0 == NULL ) || ( st_niv1 == NULL ) )
      {
         printf("Out of memory\n");
         exit(-1);
      }

      if ( graphFile )
      {
         if ( mapfilename != NULL )
         {
            posPNG = strstr( mapfilename, "png");
            if ( posPNG == NULL )
               posPNG = strstr( mapfilename, "PNG");

            posJPEG = strstr( mapfilename, "jpg");
            if ( posJPEG == NULL )
               posJPEG = strstr( mapfilename, "JPG");
            if ( posJPEG == NULL )
               posJPEG = strstr( mapfilename, "jpeg");
            if ( posJPEG == NULL )
               posJPEG = strstr( mapfilename, "JPEG");

            if ( posPNG != NULL )
            {
               im = gdImageCreateFromPng( mapfileIn );
               if ( strmapfileout == NULL )
                  mapfileInOut = fopen("world_new.png", "w"); //open a file for writing
               else
                  mapfileInOut = fopen(strmapfileout, "w"); //open a file for writing
            }
            else if ( posJPEG != NULL )
            {
               im = gdImageCreateFromJpeg( mapfileIn );
               if ( strmapfileout == NULL )
                  mapfileInOut = fopen("world_new.jpg", "w"); //open a file for writing
               else
                  mapfileInOut = fopen(strmapfileout, "w"); //open a file for writing
            }
            else
            {
               printf ("La carte graphique doit être au format png ou jpeg\n");
               graphFile = false;
            }
         }
         
         // Actions
         if ( graphFile )
         {
            unsigned int r,g,b;
            
            r = rgbColor( mycolor, 'r' );
            g = rgbColor( mycolor, 'g' );
            b = rgbColor( mycolor, 'b' );
            couleurTrace = gdImageColorAllocate(im, r,g,b);

            r = rgbColor( mplscolor, 'r' );
            g = rgbColor( mplscolor, 'g' );
            b = rgbColor( mplscolor, 'b' );
            couleurMPLS = gdImageColorAllocate(im, r,g,b);

            red   = gdImageColorAllocate(im, 255, 0, 0);
            green = gdImageColorAllocate(im, 0, 255, 0);
            blue  = gdImageColorAllocate(im, 0, 0, 255);
            // printf(" couleur : (%d) -> %d\ngreen=%d\n", mycolor, couleurTrace, green );
         }
      }
      
      xprintf ("démarrage de la lecture du répertoire %s\n", repName );
      while ((lecture = readdir (rep)))
      {
         // struct stat st;

         if ( lecture->d_name[0] == '.' )
            continue;
         
         nb_files++;
         strcpy( nfile_niv0, repName );
         strcat( nfile_niv0, "/");
         strcat( nfile_niv0, lecture->d_name);
         
         // printf("récupération des informations de %s\n", nfile_niv0);
         xprintf("récupération des informations de %s\n", nfile_niv0);

         if ( ! json_data )
         {
            if ( isFile( nfile_niv0 ))
            {
               if ( isGoodFile(nfile_niv0 ) == 0)
               {
                  // printf("comptabilise\n");
                  comptabiliseFile( nfile_niv0 );
                  
                  if ( ( graphFile ) && (usemysql ) )
                  {
                     // printf (" Ajoute des donnees RPI pour le graphe %s)\n", nfile_niv0);
                     // debug_data( xtab_tmp );
                     // printf("_______________________________________\n");
                     draw_data( xtab_tmp );
                     destroy_json_tmp(); // Supprime l'arborescence temporaire
                  }
               }
               else if ( markRejFile )
               {
                  int ret;
                  char *newname;
                  
                  newname = malloc( strlen( nfile_niv0 ) + 5 );    // reservation memoire
                  if ( newname == NULL )
                  {
                     printf("Out of memory\n");
                     exit(-1);
                  }
                  strcpy( newname, nfile_niv0 );
                  strcat( newname, ".rej" );

                  ret = rename(nfile_niv0, newname);
                  
                  free ( newname );
                  newname = NULL;
               }
            }
         }
         else
         {
            xprintf ("______________________________________________________________\n");

            // printf("Parse JSON rep : %s\n", nfile_niv0 );

            JSA = jparse( nfile_niv0 );
            if ( JSA == NULL )
            {
               // fichier en erreur :
               // "premature EOF in JSON data"
               // "unexpected 'j' on line 1 in JSON data"
               xprintf ("ignoring currupted file %s\n", nfile_niv0 );
               continue;
            }
            analyse_json( JSA );
            
            if ( isValideJsonParsing( nfile_niv0 ) )
            {
               xprintf ("++ validing values for %s\n", nfile_niv0);
               
               if ( ( graphFile ) && (usemysql ) )
               {
                  draw_data( xtab_j_all );
               }
               json_validate();
            }
            else
            {
               xprintf ("ignoring file %s\n", nfile_niv0 );
               if ( markRejFile )
               {
                  int ret;
                  char *newname;
                  
                  newname = malloc( strlen( nfile_niv0 ) + 5 );    // reservation memoire
                  if ( newname == NULL )
                  {
                     printf("Out of memory\n");
                     exit(-1);
                  }
                  strcpy( newname, nfile_niv0 );
                  strcat( newname, ".rej" );
                  
                  ret = rename(nfile_niv0, newname);
                  
                  free ( newname );
                  newname = NULL;
               }

            }
            destroy_json_tmp();
            nx_json_free( JSA );
         }
      }
      
      closedir (rep);
      free (st_niv1);
      st_niv1 = NULL;
      free (st_niv0 );
      st_niv0 = NULL;
   }
   // printf("lx - start\n");
   list_xtuple( xtab );
   i = print_occ_lien();
   xprintf ("Plus grosse occurrence des liens : %d\n", i);
   
   if ( i > 0 )
   {
      if ( redraw )
      {
         draw_occ_lien( uSage );
      }
   }
   
   // printf("lx - end\n");
   if ( im )
   {
      if ( posPNG != NULL )
         gdImagePng(im, mapfileInOut); //write the image to the file in the PNG format
      else
         gdImageJpeg(im, mapfileInOut, 90); //write the image to the file in the JPG format (90%)
   }
   

   if ( i > 0 )
   {
      xprintf(" /*/ u x y = {%d %d %d}/*/\n", i, stx, sty);

      if ( ( stx > 0 ) && ( sty > 0 ) )
      {
         FILE *mapfilelast    = NULL;

         // Ferme l'ancienne image
         if ( im != NULL )
            gdImageDestroy(im);
         im = NULL;
         
         // Ferme l'ancien fichier
         if ( mapfileIn != NULL )
            fclose(mapfileIn);
         mapfileIn = NULL;
         
         // S'il faut creer la carte "dernier point avant impact", ré-ouvre la carte "vierge"
         if ( strmapfilelast != NULL )
         {
            mapfileIn = fopen( mapfilename, "r" );
            if ( ! mapfileIn )
               xprintf("Impossible d'ouvrir le fichier %s\n", mapfilename );
         }

         if ( ( strmapfilelast != NULL ) && ( mapfileIn != NULL ) )
         {
            // Recharge la carte graphique
            if ( posPNG != NULL )
            {
               im = gdImageCreateFromPng( mapfileIn );
            }
            else if ( posJPEG != NULL )
            {
               im = gdImageCreateFromJpeg( mapfileIn );
            }
            if ( im != NULL )
               mapfilelast = fopen(strmapfilelast, "w"); //open a file for writing
         }
         else
         {
            printf ("La carte graphique doit être au format png ou jpeg\n");
         }
         
         xprintf(" /*/ Creation de la carte des derniers sauts /*/\n");
         recherche_connexions( stx, sty, im );

         if ( mapfilelast != NULL )
         {
            if ( posPNG != NULL )
            {
               gdImagePng(im, mapfilelast); //write the image to the file in the PNG format
            }
            else
               gdImageJpeg(im, mapfilelast, 90); //write the image to the file in the JPG format (90%)
            if ( mapfilelast != NULL )
               fclose(mapfilelast);
         }
      }
   }
   
   if ( matrice )
      calcule_matrice();

   sortie();
   destroy_config( &xdbConfig );
   if ( mysqlfile )
      free( mysqlfile );
   t2 = clock();
   temps = (float)(t2-t1)/CLOCKS_PER_SEC;
   if ( verbose )
      printf("temps d'analyse = %.03f (s) %u fichiers traités\n", temps, nb_files);

   return result;
}
